@@ -473,14 +473,11 @@ dm_addmap (int task, const char *target, struct multipath *mpp,
 		dm_task_set_ro(dmt);
 
 	if (task == DM_DEVICE_CREATE) {
-		prefixed_uuid = calloc(1, UUID_PREFIX_LEN +
-				       strlen(mpp->wwid) + 1);
-		if (!prefixed_uuid) {
+		if (asprintf(&prefixed_uuid, UUID_PREFIX "%s", mpp->wwid) < 0) {
 			condlog(0, "cannot create prefixed uuid : %s",
 				strerror(errno));
 			goto addout;
 		}
-		sprintf(prefixed_uuid, UUID_PREFIX "%s", mpp->wwid);
 		if (!dm_task_set_uuid(dmt, prefixed_uuid))
 			goto freeout;
 		dm_task_skip_lockfs(dmt);
@@ -611,12 +608,21 @@ int dm_addmap_reload(struct multipath *mpp, char *params, int flush)
 	return 0;
 }
 
-static int
-do_get_info(const char *name, struct dm_info *info)
+bool
+has_dm_info(const struct multipath *mpp)
+{
+	return (mpp && mpp->dmi.exists != 0);
+}
+
+int
+dm_get_info(const char *name, struct dm_info *info)
 {
 	int r = -1;
 	struct dm_task *dmt;
 
+	if (!name || !info)
+		return r;
+
 	if (!(dmt = libmp_dm_task_create(DM_DEVICE_INFO)))
 		return r;
 
@@ -646,7 +652,7 @@ int dm_map_present(const char * str)
 {
 	struct dm_info info;
 
-	return (do_get_info(str, &info) == 0);
+	return (dm_get_info(str, &info) == 0);
 }
 
 int dm_get_map(const char *name, unsigned long long *size, char **outparams)
@@ -969,7 +975,7 @@ dm_dev_t (const char * mapname, char * dev_t, int len)
 {
 	struct dm_info info;
 
-	if (do_get_info(mapname, &info) != 0)
+	if (dm_get_info(mapname, &info) != 0)
 		return 1;
 
 	if (snprintf(dev_t, len, "%i:%i", info.major, info.minor) > len)
@@ -1013,7 +1019,7 @@ dm_get_major_minor(const char *name, int *major, int *minor)
 {
 	struct dm_info info;
 
-	if (do_get_info(name, &info) != 0)
+	if (dm_get_info(name, &info) != 0)
 		return -1;
 
 	*major = info.major;
@@ -1367,7 +1373,7 @@ dm_geteventnr (const char *name)
 {
 	struct dm_info info;
 
-	if (do_get_info(name, &info) != 0)
+	if (dm_get_info(name, &info) != 0)
 		return -1;
 
 	return info.event_nr;
@@ -1378,7 +1384,7 @@ dm_is_suspended(const char *name)
 {
 	struct dm_info info;
 
-	if (do_get_info(name, &info) != 0)
+	if (dm_get_info(name, &info) != 0)
 		return -1;
 
 	return info.suspended;
@@ -1542,7 +1548,7 @@ dm_get_deferred_remove (const char * mapname)
 {
 	struct dm_info info;
 
-	if (do_get_info(mapname, &info) != 0)
+	if (dm_get_info(mapname, &info) != 0)
 		return -1;
 
 	return info.deferred_remove;
@@ -1583,32 +1589,6 @@ dm_cancel_deferred_remove (struct multipath *mpp __attribute__((unused)))
 
 #endif
 
-static struct dm_info *
-alloc_dminfo (void)
-{
-	return calloc(1, sizeof(struct dm_info));
-}
-
-int
-dm_get_info (const char * mapname, struct dm_info ** dmi)
-{
-	if (!mapname)
-		return 1;
-
-	if (!*dmi)
-		*dmi = alloc_dminfo();
-
-	if (!*dmi)
-		return 1;
-
-	if (do_get_info(mapname, *dmi) != 0) {
-		free(*dmi);
-		*dmi = NULL;
-		return 1;
-	}
-	return 0;
-}
-
 struct rename_data {
 	const char *old;
 	char *new;
